"""Real-time code monitoring for Windsurf IDE integration."""

import os
import sys
import time
import logging
import threading
import torch
from watchdog.observers.polling import PollingObserver as Observer
from watchdog.events import FileSystemEventHandler

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
    handlers=[
        logging.StreamHandler()
    ]
)

logger = logging.getLogger(__name__)

class CodeChangeHandler(FileSystemEventHandler):
    def on_modified(self, event):
        if event.is_directory:
            return
            
        if event.src_path.endswith(('.py', '.md', '.json')):
            logger.info(f"Change detected in: {event.src_path}")

class ResourceMonitor:
    def __init__(self):
        self.gpu_metrics = {
            'memory_usage': [],
            'utilization': []
        }
    
    def track_gpu(self):
        while True:
            if torch.cuda.is_available():
                self.gpu_metrics['memory_usage'].append(
                    torch.cuda.memory_allocated()/torch.cuda.max_memory_allocated()
                )
                self.gpu_metrics['utilization'].append(
                    torch.cuda.utilization()
                )
            time.sleep(1)

def start_monitoring():
    # Get the project root directory
    current_dir = os.path.dirname(os.path.abspath(__file__))
    project_root = os.path.dirname(os.path.dirname(os.path.dirname(current_dir)))
    
    logger.info(f"Starting monitoring in: {project_root}")
    
    event_handler = CodeChangeHandler()
    observer = Observer()
    observer.schedule(event_handler, project_root, recursive=True)
    observer.start()
    
    monitor = ResourceMonitor()
    threading.Thread(target=monitor.track_gpu, daemon=True).start()
    
    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
        observer.join()

if __name__ == "__main__":
    start_monitoring()
