import pytest
import pandas as pd
import numpy as np
from datetime import datetime, timezone
from unittest.mock import Mock, patch
from app.core.charting.engine import ChartEngine
from app.core.charting.stream import ChartDataStream


class MockFigure:
    """Mock for Plotly Figure."""

    def __init__(self):
        self.data = []
        self.layout = {}

    def add_trace(self, trace):
        self.data.append(trace)
        return self

    def update_layout(self, **kwargs):
        self.layout.update(kwargs)
        return self


@pytest.fixture
def mock_plotly(monkeypatch):
    """Mock Plotly's Figure class."""

    def mock_figure(*args, **kwargs):
        return MockFigure()

    monkeypatch.setattr("plotly.graph_objects.Figure", mock_figure)
    return mock_figure


@pytest.fixture
def sample_ohlcv_data():
    """Create sample OHLCV data."""
    dates = pd.date_range(
        start="2025-01-01", end="2025-01-10", freq="D", tz=timezone.utc
    )
    data = {
        "open": np.random.uniform(100, 110, len(dates)),
        "high": np.random.uniform(110, 120, len(dates)),
        "low": np.random.uniform(90, 100, len(dates)),
        "close": np.random.uniform(100, 110, len(dates)),
        "volume": np.random.uniform(1000, 2000, len(dates)),
    }
    return pd.DataFrame(data, index=dates)


@pytest.fixture
def chart_engine():
    """Create ChartEngine instance."""
    return ChartEngine()


class MockRedisClient:
    """Mock Redis client for testing."""

    def __init__(self):
        self.connected = True
        self.subscriptions = {}
        self.data = {}

    async def connect(self):
        self.connected = True

    async def close(self):
        self.connected = False

    async def subscribe(self, channel):
        self.subscriptions[channel] = []
        return True

    async def publish(self, channel, message):
        if channel in self.subscriptions:
            self.subscriptions[channel].append(message)
        return True

    async def get(self, key):
        return self.data.get(key)

    async def set(self, key, value):
        self.data[key] = value
        return True


@pytest.fixture
def mock_redis_client():
    """Create a mock Redis client for testing."""
    return MockRedisClient()


def test_create_candlestick(sample_ohlcv_data, mock_plotly):
    """Test candlestick chart creation."""
    engine = ChartEngine()
    fig = engine.create_candlestick(sample_ohlcv_data)
    assert isinstance(fig, MockFigure)
    assert len(fig.data) > 0  # Should have at least one trace


def test_add_indicator(chart_engine, sample_ohlcv_data, mock_plotly):
    """Test adding technical indicators to chart."""
    fig = chart_engine.create_candlestick(sample_ohlcv_data, indicators=["sma"])
    assert isinstance(fig, MockFigure)
    assert len(fig.data) > 1  # Should have candlestick and SMA traces


def test_validate_ohlc(chart_engine, sample_ohlcv_data):
    """Test OHLC data validation."""
    assert chart_engine.validate_ohlc(sample_ohlcv_data) == True

    # Test with invalid data
    invalid_data = sample_ohlcv_data.copy()
    invalid_data.loc[invalid_data.index[0], "close"] = None
    assert chart_engine.validate_ohlc(invalid_data) == False


@pytest.mark.asyncio
async def test_chart_data_stream():
    """Test chart data stream initialization."""
    mock_redis = MockRedisClient()
    stream = ChartDataStream(redis_client=mock_redis)
    await stream.connect()
    assert stream._connected == True


@pytest.mark.asyncio
async def test_chart_stream_subscription(mock_redis_client):
    """Test chart data stream subscription."""
    stream = ChartDataStream(redis_client=mock_redis_client)
    symbol = "AAPL"
    interval = "1m"

    # Initialize stream
    await stream.connect()
    assert stream._connected == True

    # Test subscription
    await stream.subscribe(symbol, interval)
    stream_id = f"{symbol}_{interval}"
    assert stream_id in stream.active_streams


def test_stream_cleanup():
    """Test stream cleanup on close."""
    mock_redis = MockRedisClient()
    stream = ChartDataStream(redis_client=mock_redis)
    assert hasattr(stream, "cleanup")


@pytest.mark.asyncio
async def test_redis_connection_handling():
    """Test Redis connection error handling in ChartDataStream."""
    # Test with invalid Redis URL
    stream = ChartDataStream(redis_url="redis://invalid_host:6379")

    with pytest.raises(ConnectionError):
        await stream.connect()

    # Test subscription without connection
    with pytest.raises(ConnectionError):
        await stream.subscribe("BTCUSDT", "1m")
