"""Test strategy execution and optimization."""

import asyncio
import json
import os
from datetime import datetime

import numpy as np
import pandas as pd
import pytest

from app.core.security import SecurityException
from app.core.strategy_service import StrategyOrchestrator


@pytest.fixture
def valid_strategy_dna():
    """Create a valid strategy DNA for testing."""
    return {
        "id": "test_strategy_001",
        "name": "Test Strategy",
        "symbol": "BTCUSDT",
        "timeframe": "1d",
        "user_id": "test_user",
        "parameters": {"risk_per_trade": 0.02, "stop_loss": 100},
        "signature": "d460d440ff4f171fea3edcc68b3e073741a7add3b789d136347e5a27f4cfd32c",
        "metrics": {"sharpe_ratio": 1.5, "max_drawdown": 0.2},
    }


@pytest.fixture
def strategy_orchestrator(monkeypatch):
    """Create a strategy orchestrator with mocked dependencies."""
    orchestrator = StrategyOrchestrator()

    # Mock verify_strategy_signature
    def mock_verify(signature, user_id):
        if (
            signature
            == "d460d440ff4f171fea3edcc68b3e073741a7add3b789d136347e5a27f4cfd32c"
        ):
            return {
                "id": "test_strategy_001",
                "name": "Test Strategy",
                "symbol": "BTCUSDT",
                "timeframe": "1d",
                "user_id": "test_user",
                "parameters": {"risk_per_trade": 0.02, "stop_loss": 100},
                "lookback": 100,
                "metrics": {"sharpe_ratio": 1.5, "max_drawdown": 0.2},
            }
        return None

    monkeypatch.setattr(orchestrator, "verify_strategy_signature", mock_verify)

    # Mock market cache
    def mock_get_data(symbol, lookback):
        dates = pd.date_range(start="2025-01-01", end="2025-01-10", freq="1D")
        data = {
            "open": np.random.uniform(100, 110, len(dates)),
            "high": np.random.uniform(110, 120, len(dates)),
            "low": np.random.uniform(90, 100, len(dates)),
            "close": np.random.uniform(100, 110, len(dates)),
            "volume": np.random.uniform(1000, 2000, len(dates)),
        }
        return pd.DataFrame(data, index=dates)

    monkeypatch.setattr(orchestrator.market_cache, "get_data", mock_get_data)

    # Mock optimizer
    def mock_forward(market_data, strategy_params):
        return strategy_params

    monkeypatch.setattr(orchestrator.optimizer, "forward", mock_forward)

    return orchestrator


@pytest.mark.asyncio
async def test_strategy_execution(strategy_orchestrator, valid_strategy_dna):
    """Test strategy execution with valid DNA"""
    result = await strategy_orchestrator.execute_strategy(valid_strategy_dna)
    assert result is not None
    assert result["id"] == valid_strategy_dna["id"]
    assert result["timeframe"] == valid_strategy_dna["timeframe"]
    assert result["parameters"]["risk_per_trade"] == 0.02


@pytest.mark.asyncio
async def test_invalid_signature(strategy_orchestrator, valid_strategy_dna):
    """Test strategy execution with invalid signature"""
    # Corrupt the signature
    valid_strategy_dna["signature"] = "invalid_signature"

    with pytest.raises(SecurityException):
        await strategy_orchestrator.execute_strategy(valid_strategy_dna)


@pytest.mark.asyncio
async def test_results_storage(strategy_orchestrator, valid_strategy_dna):
    """Test strategy results storage"""
    result = await strategy_orchestrator.execute_strategy(valid_strategy_dna)

    # Verify results are stored correctly
    assert result["id"] == valid_strategy_dna["id"]
    assert result["timeframe"] == valid_strategy_dna["timeframe"]
    assert result["parameters"]["risk_per_trade"] == 0.02
    assert "execution_time" in result
    assert "results" in result
