import pytest
from fastapi.testclient import TestClient
from fastapi.websockets import WebSocket
import asyncio
import json
from datetime import datetime
from app.core.websocket_manager import WebSocketManager
from main import app


@pytest.fixture
def websocket_manager():
    """Create a WebSocket manager instance for testing"""
    return WebSocketManager()


@pytest.fixture
def test_client():
    """Create a FastAPI test client"""
    return TestClient(app)


class MockWebSocket:
    """Mock WebSocket for testing"""

    def __init__(self):
        self.sent_messages = []
        self.closed = False

    async def accept(self):
        pass

    async def send_json(self, data: dict):
        self.sent_messages.append(data)

    async def close(self):
        self.closed = True


@pytest.mark.asyncio
async def test_websocket_connection(websocket_manager):
    """Test WebSocket connection and disconnection"""
    ws = MockWebSocket()

    # Test connection
    await websocket_manager.connect(ws, "market_data")
    assert ws in websocket_manager.active_connections["market_data"]

    # Test disconnection
    await websocket_manager.disconnect(ws, "market_data")
    assert ws not in websocket_manager.active_connections["market_data"]


@pytest.mark.asyncio
async def test_market_data_broadcast(websocket_manager):
    """Test broadcasting market data to connected clients"""
    ws1 = MockWebSocket()
    ws2 = MockWebSocket()

    # Connect two clients
    await websocket_manager.connect(ws1, "market_data")
    await websocket_manager.connect(ws2, "market_data")

    # Broadcast test data
    test_data = {"symbol": "BTC/USD", "price": 50000, "volume": 100}

    await websocket_manager.broadcast_market_data(test_data)

    # Verify both clients received the data with UTC timestamp
    for ws in [ws1, ws2]:
        assert len(ws.sent_messages) == 1
        received = ws.sent_messages[0]
        assert received["symbol"] == test_data["symbol"]
        assert received["price"] == test_data["price"]
        assert received["volume"] == test_data["volume"]
        assert "timestamp" in received

        # Verify timestamp format
        timestamp = datetime.fromisoformat(received["timestamp"])
        assert timestamp.tzinfo is not None


@pytest.mark.asyncio
async def test_strategy_updates(websocket_manager):
    """Test strategy execution updates"""
    ws = MockWebSocket()
    await websocket_manager.connect(ws, "strategy_updates")

    strategy_id = "test_strategy_001"
    update = {"status": "running", "progress": 50, "metrics": {"sharpe_ratio": 1.5}}

    await websocket_manager.broadcast_strategy_update(strategy_id, update)

    # Verify update was received
    assert len(ws.sent_messages) == 1
    received = ws.sent_messages[0]
    assert received["strategy_id"] == strategy_id
    assert received["update"] == update
    assert "timestamp" in received


@pytest.mark.asyncio
async def test_client_disconnection_handling(websocket_manager):
    """Test handling of client disconnections"""

    class FailingWebSocket(MockWebSocket):
        async def send_json(self, data: dict):
            raise ConnectionError("Client disconnected")

    ws = FailingWebSocket()
    await websocket_manager.connect(ws, "market_data")

    # Try to send data, should handle the error gracefully
    await websocket_manager.broadcast_market_data({"test": "data"})

    # Verify client was removed from active connections
    assert ws not in websocket_manager.active_connections["market_data"]
