"""Tests for research agent clients."""

import pytest
import os
from datetime import datetime, timezone
from unittest.mock import AsyncMock, patch
from research_agent.clients.market_data import GlassnodeClient, CryptoQuantAPI
from research_agent.clients.news_client import NewsAPIClient


@pytest.fixture
def mock_glassnode_response():
    """Mock Glassnode API response."""
    mock = AsyncMock()
    mock.json.return_value = {"value": 123.45}
    mock.raise_for_status = AsyncMock()
    return mock


@pytest.fixture
def mock_cryptoquant_response():
    """Mock CryptoQuant API response."""
    mock = AsyncMock()
    mock.json.return_value = {"inflow": 100.0, "outflow": 80.0}
    mock.raise_for_status = AsyncMock()
    return mock


@pytest.fixture
def mock_news_response():
    """Mock News API response."""
    mock = AsyncMock()
    mock.json.return_value = {
        "articles": [
            {
                "title": "Test Article",
                "description": "Test Description",
                "url": "https://test.com",
                "publishedAt": "2025-02-07T00:00:00Z",
                "source": {"name": "Test Source"},
            }
        ]
    }
    mock.raise_for_status = AsyncMock()
    return mock


@pytest.fixture
def glassnode_client():
    return GlassnodeClient()


@pytest.fixture
def cryptoquant_client():
    return CryptoQuantAPI()


@pytest.fixture
def news_client():
    return NewsAPIClient()


@pytest.mark.asyncio
@patch("httpx.AsyncClient")
async def test_glassnode_metrics(
    mock_client_class, glassnode_client, mock_glassnode_response
):
    """Test fetching Glassnode metrics."""
    mock_client_class.return_value.__aenter__.return_value.get.return_value = (
        mock_glassnode_response
    )

    metrics = await glassnode_client.get_metrics("BTC")
    assert isinstance(metrics, dict)
    assert all(key in metrics for key in ["active_addresses", "exchange_flows", "sopr"])


@pytest.mark.asyncio
@patch("httpx.AsyncClient")
async def test_cryptoquant_flows(
    mock_client_class, cryptoquant_client, mock_cryptoquant_response
):
    """Test fetching CryptoQuant exchange flows."""
    mock_client_class.return_value.__aenter__.return_value.get.return_value = (
        mock_cryptoquant_response
    )

    flows = await cryptoquant_client.get_exchange_flows("BTC")
    assert isinstance(flows, dict)
    assert "inflow" in flows
    assert "outflow" in flows
    assert "net_flow" in flows
    assert flows["net_flow"] == flows["inflow"] - flows["outflow"]


@pytest.mark.asyncio
@patch("httpx.AsyncClient")
async def test_news_sentiment(mock_client_class, news_client, mock_news_response):
    """Test news sentiment analysis."""
    mock_client_class.return_value.__aenter__.return_value.get.return_value = (
        mock_news_response
    )

    sentiment = await news_client.get_market_sentiment("bitcoin")
    assert isinstance(sentiment, dict)
    assert "sentiment_score" in sentiment
    assert "article_count" in sentiment
    assert "timestamp" in sentiment


@pytest.mark.asyncio
@patch("httpx.AsyncClient")
async def test_news_latest(mock_client_class, news_client, mock_news_response):
    """Test fetching latest news."""
    mock_client_class.return_value.__aenter__.return_value.get.return_value = (
        mock_news_response
    )

    articles = await news_client.get_latest(["bitcoin", "crypto"])
    assert isinstance(articles, list)
    assert len(articles) == 1
    article = articles[0]
    assert article["title"] == "Test Article"
    assert article["description"] == "Test Description"
    assert article["url"] == "https://test.com"
    assert article["published_at"] == "2025-02-07T00:00:00Z"
    assert article["source"] == "Test Source"
