import unittest
import pandas as pd
import numpy as np
from datetime import datetime, timezone, timedelta
from app.core.data_management import MarketDataCache


class TestMarketDataCache(unittest.TestCase):
    def setUp(self):
        self.cache = MarketDataCache()
        self.test_data = pd.DataFrame(
            {
                "open": np.random.uniform(100, 110, 10),
                "high": np.random.uniform(110, 120, 10),
                "low": np.random.uniform(90, 100, 10),
                "close": np.random.uniform(100, 110, 10),
                "volume": np.random.uniform(1000, 2000, 10),
            },
            index=pd.date_range(
                start=datetime.now(timezone.utc), periods=10, freq="1D"
            ),
        )

    def test_cache_retrieval(self):
        """Test basic cache functionality."""
        symbol = "AAPL"
        self.cache.update_data(symbol, self.test_data)

        # Test data retrieval
        cached_data = self.cache.get_data(symbol)
        pd.testing.assert_frame_equal(cached_data, self.test_data)

        # Test lookback
        lookback_data = self.cache.get_data(symbol, lookback=5)
        self.assertEqual(len(lookback_data), 5)
        pd.testing.assert_frame_equal(lookback_data, self.test_data.tail(5))

    def test_time_slicing(self):
        """Test time-based data slicing."""
        symbol = "AAPL"
        self.cache.update_data(symbol, self.test_data)

        # Test time slice
        start = self.test_data.index[2]
        end = self.test_data.index[5]
        sliced_data = self.cache.slice_time(symbol, start, end)
        pd.testing.assert_frame_equal(sliced_data, self.test_data[start:end])

        # Test with timezone-naive datetimes
        naive_start = start.replace(tzinfo=None)
        naive_end = end.replace(tzinfo=None)
        sliced_data = self.cache.slice_time(symbol, naive_start, naive_end)
        pd.testing.assert_frame_equal(sliced_data, self.test_data[start:end])

    def test_redis_persistence(self):
        """Test Redis persistence."""

        class MockRedis:
            def __init__(self):
                self.data = {}

            async def set(self, key, value):
                self.data[key] = value
                return True

            async def get(self, key):
                return self.data.get(key)

        # Create cache with mock Redis
        mock_redis = MockRedis()
        redis_cache = MarketDataCache(redis_client=mock_redis)
        symbol = "AAPL"

        # Round microseconds to avoid precision issues
        test_data = self.test_data.copy()
        test_data.index = test_data.index.floor("ms")
        redis_cache.update_data(symbol, test_data)

        # Test persistence
        import asyncio

        asyncio.run(redis_cache.persist())

        # Test loading
        new_cache = MarketDataCache(redis_client=mock_redis)
        asyncio.run(new_cache.load(symbol))

        # Compare with microseconds floored
        loaded_data = new_cache.get_data(symbol)
        loaded_data.index = loaded_data.index.floor("ms")
        pd.testing.assert_frame_equal(loaded_data, test_data)


if __name__ == "__main__":
    unittest.main()
