import pytest
import torch
import torch.nn as nn
import numpy as np
from SYS_RESEARCH.training.distributed_trainer import DistributedTrainer
from SYS_RESEARCH.training.custom_dataset import FinancialDataset

class SimpleModel(nn.Module):
    def __init__(self):
        super().__init__()
        self.network = nn.Sequential(
            nn.Linear(10, 64),
            nn.ReLU(),
            nn.Linear(64, 1)
        )
    
    def forward(self, x):
        return self.network(x)

@pytest.fixture
def sample_dataset():
    # Create sample data
    X = np.random.randn(100, 10)
    y = np.random.randn(100, 1)
    return FinancialDataset.from_numpy(X, y)

def test_dataset_creation(sample_dataset):
    assert len(sample_dataset) == 100
    sample_x, sample_y = sample_dataset[0]
    assert isinstance(sample_x, torch.Tensor)
    assert isinstance(sample_y, torch.Tensor)
    assert sample_x.shape == torch.Size([10])
    assert sample_y.shape == torch.Size([1])

@pytest.mark.skipif(not torch.cuda.is_available(), reason="CUDA not available")
def test_distributed_trainer():
    model = SimpleModel()
    trainer = DistributedTrainer(model, rank=0, world_size=1)
    assert isinstance(trainer.model, nn.parallel.DistributedDataParallel)
    assert trainer.device == torch.device("cuda:0" if torch.cuda.is_available() else "cpu")

def test_data_normalization():
    X = np.random.randn(100, 10) * 10 + 5  # Non-normalized data
    normalized_X = FinancialDataset.normalize_data(X)
    assert np.abs(normalized_X.mean()) < 1e-5  # Mean should be close to 0
    assert np.abs(normalized_X.std() - 1) < 1e-5  # Std should be close to 1
