"""Tests for event bus system."""

import pytest
import asyncio
import fakeredis.aioredis
from datetime import datetime, timezone
from app.core.event_bus import EventBus, Event, EventType
import pandas as pd


@pytest.fixture
async def event_bus():
    """Create and configure event bus for testing."""
    # Use fakeredis instead of real Redis
    fake_redis = fakeredis.aioredis.FakeRedis()
    bus = EventBus(
        redis_url="redis://localhost",  # URL doesn't matter for fakeredis
        stream_name="test_events",
        consumer_group="test_group",
        redis_client=fake_redis,  # Pass the fake client
    )
    await bus.connect()
    yield bus
    await bus.aclose()


@pytest.mark.asyncio
async def test_event_creation():
    """Test event creation and serialization."""
    event = Event(
        type=EventType.STRATEGY_SUBMITTED, payload={"strategy_id": "123"}, source="test"
    )

    assert event.type == EventType.STRATEGY_SUBMITTED
    assert event.payload["strategy_id"] == "123"
    assert event.source == "test"

    # Test serialization
    json_str = event.to_json()
    restored = Event.from_json(json_str)
    assert restored.type == event.type
    assert restored.payload == event.payload
    assert restored.source == event.source


@pytest.mark.asyncio
async def test_event_signature(event_bus):
    """Test event signing and verification."""
    event = Event(
        type=EventType.STRATEGY_SUBMITTED, payload={"test": "data"}, source="test"
    )

    # Sign event
    signature = event_bus._sign_event(event)

    # Verify signature
    assert event_bus._verify_signature(event.to_json(), signature)

    # Test invalid signature
    assert not event_bus._verify_signature(event.to_json(), "invalid_signature")


@pytest.mark.asyncio
async def test_publish_subscribe(event_bus):
    """Test publishing and subscribing to events."""
    received_events = []

    async def test_handler(event: Event):
        received_events.append(event)

    # Subscribe to events
    await event_bus.subscribe(EventType.STRATEGY_SUBMITTED, test_handler)

    # Start processing in background
    process_task = asyncio.create_task(event_bus.start_processing())

    # Publish test event
    test_event = Event(
        type=EventType.STRATEGY_SUBMITTED, payload={"test": "data"}, source="test"
    )
    await event_bus.publish(test_event)

    # Wait for processing
    await asyncio.sleep(1)

    # Stop processing
    process_task.cancel()

    # Verify event was received
    assert len(received_events) == 1
    received = received_events[0]
    assert received.type == test_event.type
    assert received.payload == test_event.payload
    assert received.source == test_event.source


@pytest.mark.asyncio
async def test_multiple_handlers(event_bus):
    """Test multiple handlers for same event type."""
    handler1_called = asyncio.Event()
    handler2_called = asyncio.Event()

    async def handler1(event: Event):
        handler1_called.set()

    async def handler2(event: Event):
        handler2_called.set()

    # Subscribe both handlers
    await event_bus.subscribe(EventType.STRATEGY_SUBMITTED, handler1)
    await event_bus.subscribe(EventType.STRATEGY_SUBMITTED, handler2)

    # Start processing
    process_task = asyncio.create_task(event_bus.start_processing())

    # Publish event
    await event_bus.publish(
        Event(type=EventType.STRATEGY_SUBMITTED, payload={"test": "data"})
    )

    # Wait for handlers
    await asyncio.wait_for(
        asyncio.gather(handler1_called.wait(), handler2_called.wait()), timeout=2
    )

    # Cleanup
    process_task.cancel()

    assert handler1_called.is_set()
    assert handler2_called.is_set()


@pytest.mark.asyncio
async def test_error_handling(event_bus):
    """Test error handling in event processing."""
    error_logged = asyncio.Event()

    async def failing_handler(event: Event):
        raise ValueError("Test error")

    # Patch logger to detect error
    def log_error(msg):
        if "Handler error" in msg:
            error_logged.set()

    event_bus.logger.error = log_error

    # Subscribe failing handler
    await event_bus.subscribe(EventType.STRATEGY_SUBMITTED, failing_handler)

    # Start processing
    process_task = asyncio.create_task(event_bus.start_processing())

    # Publish event
    await event_bus.publish(
        Event(type=EventType.STRATEGY_SUBMITTED, payload={"test": "data"})
    )

    # Wait for error to be logged
    await asyncio.wait_for(error_logged.wait(), timeout=2)

    # Cleanup
    process_task.cancel()

    assert error_logged.is_set()


@pytest.mark.asyncio
async def test_timestamp_serialization():
    """Test proper serialization of various timestamp formats."""
    # Test pandas Timestamp
    pd_timestamp = pd.Timestamp.now(tz=timezone.utc)
    event = Event(
        type=EventType.MARKET_DATA,
        payload={"timestamp": pd_timestamp, "price": 100.0},
        source="test",
    )

    json_str = event.to_json()
    restored = Event.from_json(json_str)

    # Verify pandas Timestamp serialization
    assert isinstance(restored.payload["timestamp"], str)
    assert pd.Timestamp(restored.payload["timestamp"]).tz == timezone.utc

    # Test datetime with timezone
    dt_timestamp = datetime.now(timezone.utc)
    event = Event(
        type=EventType.MARKET_DATA,
        payload={"timestamp": dt_timestamp, "price": 100.0},
        source="test",
    )

    json_str = event.to_json()
    restored = Event.from_json(json_str)

    # Verify datetime serialization
    assert isinstance(restored.payload["timestamp"], str)
    assert datetime.fromisoformat(restored.payload["timestamp"]).tzinfo == timezone.utc

    # Test DataFrame with timestamp index
    df = pd.DataFrame(
        {"price": [100.0, 101.0, 102.0]},
        index=pd.date_range(start="2025-01-01", periods=3, tz=timezone.utc),
    )
    event = Event(type=EventType.MARKET_DATA, payload={"data": df}, source="test")

    json_str = event.to_json()
    restored = Event.from_json(json_str)

    # Verify DataFrame serialization
    assert isinstance(restored.payload["data"], dict)
    restored_df = pd.DataFrame.from_dict(restored.payload["data"], orient="index")
    assert all(isinstance(idx, str) for idx in restored_df.index)
    assert all("Z" in idx or "+00:00" in idx for idx in restored_df.index)
