"""Tests for parallel backtesting engine."""

import unittest
import numpy as np
import pandas as pd
from datetime import datetime, timezone, timedelta
from app.core.parallel_engine import ParallelBacktest, BacktestResult


def create_test_data(n_days: int = 100) -> pd.DataFrame:
    """Create test market data"""
    dates = [datetime.now(timezone.utc) - timedelta(days=x) for x in range(n_days)]

    data = pd.DataFrame(
        {
            "open": np.random.randn(n_days).cumsum(),
            "high": np.random.randn(n_days).cumsum(),
            "low": np.random.randn(n_days).cumsum(),
            "close": np.random.randn(n_days).cumsum(),
            "volume": np.random.randint(1000, 10000, n_days),
        },
        index=dates,
    )

    # Ensure high/low are actually high/low
    data["high"] = data[["open", "close"]].max(axis=1) + abs(np.random.randn(n_days))
    data["low"] = data[["open", "close"]].min(axis=1) - abs(np.random.randn(n_days))
    return data


def simple_strategy(row: pd.Series) -> float:
    """Simple test strategy"""
    return 1.0 if row["close"] > row["open"] else -1.0


class TestParallelBacktest(unittest.TestCase):
    def setUp(self):
        """Set up test data and engine"""
        self.data = create_test_data()
        self.engine = ParallelBacktest(n_jobs=2)
        self.strategies = [
            {
                "id": f"test_{i}",
                "logic": simple_strategy,
                "parameters": {"test_param": i},
            }
            for i in range(10)
        ]

    def test_single_backtest(self):
        """Test single strategy backtest"""
        results = self.engine._backtest_single(self.strategies[0], self.data)
        self.assertIsInstance(results, BacktestResult)
        self.assertEqual(len(results.returns), len(self.data))
        self.assertEqual(len(results.positions), len(self.data))

    def test_parallel_execution(self):
        """Test parallel execution of multiple strategies"""
        results = self.engine.run(self.strategies, self.data)
        self.assertEqual(len(results), len(self.strategies))
        self.assertTrue(all(isinstance(r, BacktestResult) for r in results))

    def test_metrics_calculation(self):
        """Test that metrics are calculated correctly"""
        results = self.engine.run(self.strategies, self.data)
        for result in results:
            self.assertIn("total_return", result.metrics)
            self.assertIn("sharpe_ratio", result.metrics)
            self.assertIn("max_drawdown", result.metrics)

    def test_parameter_preservation(self):
        """Test that strategy parameters are preserved"""
        results = self.engine.run(self.strategies, self.data)
        for i, result in enumerate(results):
            self.assertEqual(
                result.parameters["test_param"],
                self.strategies[i]["parameters"]["test_param"],
            )

    def test_empty_strategies(self):
        """Test handling of empty strategy list"""
        results = self.engine.run([], self.data)
        self.assertEqual(len(results), 0)

    def test_batch_processing(self):
        """Test batch processing with different sizes"""
        for batch_size in [2, 5, 10]:
            engine = ParallelBacktest(n_jobs=2, batch_size=batch_size)
            results = engine.run(self.strategies, self.data)
            self.assertEqual(len(results), len(self.strategies))


if __name__ == "__main__":
    unittest.main()
