"""
Tests for PineScript converter
"""

import pytest
from converters import PineScriptConverter, PineScriptConfig, TimeFrame


@pytest.fixture
def converter():
    """Create a PineScriptConverter instance"""
    return PineScriptConverter()


def test_basic_conversion(converter):
    """Test basic PineScript to Python conversion"""
    pine_script = """
    //@version=5
    strategy("Simple MA Cross", overlay=true)
    
    fast_length = input(10, "Fast Length")
    slow_length = input(20, "Slow Length")
    
    fast_ma = sma(close, fast_length)
    slow_ma = sma(close, slow_length)
    
    buy = crossover(fast_ma, slow_ma)
    sell = crossunder(fast_ma, slow_ma)
    
    if (buy)
        strategy.entry("buy", strategy.long)
    if (sell)
        strategy.close("buy")
    """

    python_code = converter.convert(pine_script)

    # Check for essential components
    assert "class SimpleMAcross:" in python_code
    assert "def calculate_signals" in python_code
    assert "ta.SMA" in python_code
    assert "crossover" in python_code
    assert "crossunder" in python_code


def test_advanced_indicators(converter):
    """Test conversion of advanced technical indicators"""
    pine_script = """
    //@version=5
    strategy("Advanced Indicators")
    
    // Momentum
    rsi_val = rsi(close, 14)
    macd_val, signal, hist = macd(close, 12, 26, 9)
    stoch_k, stoch_d = stoch(close, high, low, 14)
    
    // Volatility
    atr_val = atr(14)
    upper, middle, lower = bbands(close, 20, 2)
    keltner_upper, keltner_middle, keltner_lower = keltner(close, 20, 2)
    
    // Volume
    obv_val = obv
    ad_val = ad
    adosc_val = adosc(3, 10)
    """

    python_code = converter.convert(pine_script)

    # Check for indicator conversions
    assert "ta.RSI" in python_code
    assert "ta.MACD" in python_code
    assert "ta.STOCH" in python_code
    assert "ta.ATR" in python_code
    assert "ta.BBANDS" in python_code
    assert "ta.OBV" in python_code
    assert "ta.AD" in python_code
    assert "ta.ADOSC" in python_code


def test_price_action_patterns(converter):
    """Test conversion of price action patterns"""
    pine_script = """
    //@version=5
    strategy("Price Action Patterns")
    
    // Pivot points
    pivot_high = pivothigh(high, 2, 2)
    pivot_low = pivotlow(low, 2, 2)
    
    // Pattern detection
    rising_3 = rising(close, 3)
    falling_3 = falling(close, 3)
    
    // Value tracking
    last_pivot = valuewhen(pivot_high, high, 0)
    bars_since_high = barssince(pivot_high)
    """

    python_code = converter.convert(pine_script)

    # Check for pattern detection
    assert "ta.PIVOTHIGH" in python_code
    assert "ta.PIVOTLOW" in python_code
    assert "_convert_rising" in python_code
    assert "_convert_falling" in python_code
    assert "_convert_valuewhen" in python_code
    assert "_convert_barssince" in python_code


def test_complex_strategy_logic(converter):
    """Test conversion of complex strategy logic"""
    pine_script = """
    //@version=5
    strategy("Complex Strategy", overlay=true, pyramiding=2)
    
    // Risk management
    atr_length = input(14, "ATR Length")
    atr_multiplier = input(2.0, "ATR Multiplier")
    risk_percent = input(1.0, "Risk Per Trade %") * 0.01
    
    // Indicators
    atr_val = atr(atr_length)
    [macd_val, signal, hist] = macd(close, 12, 26, 9)
    [rsi_val] = rsi(close, 14)
    
    // Entry conditions
    long_entry = macd_val > signal and rsi_val < 30
    short_entry = macd_val < signal and rsi_val > 70
    
    // Position sizing
    stop_distance = atr_val * atr_multiplier
    position_size = (strategy.equity * risk_percent) / stop_distance
    
    // Entry rules
    if (long_entry and strategy.position_size <= 0)
        strategy.entry("Long", strategy.long, qty=position_size)
    
    if (short_entry and strategy.position_size >= 0)
        strategy.entry("Short", strategy.short, qty=position_size)
    
    // Exit rules
    trailing_stop = valuewhen(long_entry, low, 0) - stop_distance
    trailing_stop := rising(trailing_stop[1], stop_distance)
    
    if (strategy.position_size > 0 and close < trailing_stop)
        strategy.close("Long")
    """

    python_code = converter.convert(pine_script)

    # Check for complex features
    assert "risk_percent" in python_code
    assert "position_size" in python_code
    assert "trailing_stop" in python_code
    assert "strategy.position_size" in python_code
    assert "strategy.equity" in python_code


def test_timeframe_config():
    """Test different timeframe configurations"""
    config = PineScriptConfig(
        timeframe=TimeFrame.MIN5, overlay=True, precision=8, max_bars_back=5000
    )

    converter = PineScriptConverter(config)
    assert converter.config.timeframe == TimeFrame.MIN5


def test_plotting_functions(converter):
    """Test conversion of plotting functions"""
    pine_script = """
    //@version=5
    strategy("Plot Test", overlay=true)
    
    ma = sma(close, 20)
    upper = ma + atr(14)
    lower = ma - atr(14)
    
    plot(ma, "MA", color=color.blue)
    plot(upper, "Upper", color=color.green)
    plot(lower, "Lower", color=color.red)
    
    fill(upper, lower, color=color.new(color.green, 90))
    hline(0, "Zero Line", color=color.gray)
    bgcolor(close > ma ? color.green : color.red, transp=90)
    """

    python_code = converter.convert(pine_script)

    # Check for plotting functions
    assert "_convert_plot" in python_code
    assert "_convert_fill" in python_code
    assert "_convert_hline" in python_code
    assert "_convert_bgcolor" in python_code


def test_error_handling():
    """Test error handling for invalid PineScript"""
    converter = PineScriptConverter()

    with pytest.raises(ValueError, match="Invalid PineScript"):
        converter.convert("invalid pine script")

    with pytest.raises(ValueError, match="Empty input"):
        converter.convert("")

    with pytest.raises(ValueError, match="Version not specified"):
        converter.convert("strategy('Test')")


def test_cli(tmp_path):
    """Test command-line interface"""
    from converters.cli import main
    import sys
    from unittest.mock import patch

    # Create test input file
    input_file = tmp_path / "test.pine"
    input_file.write_text(
        """
    //@version=5
    strategy("Test Strategy")
    """
    )

    # Test basic conversion
    with patch.object(sys, "argv", ["converter", str(input_file)]):
        main()
        assert (tmp_path / "test.py").exists()

    # Test with config file
    config_file = tmp_path / "config.json"
    config_file.write_text(
        """
    {
        "timeframe": "5",
        "overlay": true,
        "risk_per_trade": 0.01
    }
    """
    )

    with patch.object(
        sys,
        "argv",
        [
            "converter",
            str(input_file),
            "-c",
            str(config_file),
            "-o",
            str(tmp_path / "output.py"),
        ],
    ):
        main()
        assert (tmp_path / "output.py").exists()


def test_conversion_context(converter):
    """Test conversion context management"""
    pine_script = """
    //@version=5
    strategy("Context Test")
    
    f_rsi_cross(src, length) =>
        r = rsi(src, length)
        c = cross(r, 50)
        [r, c]
    
    [rsi_val, cross_50] = f_rsi_cross(close, 14)
    
    if (cross_50)
        strategy.entry("RSI Cross", strategy.long)
    """

    python_code = converter.convert(pine_script)

    # Check context tracking
    assert converter.context.in_function is False
    assert len(converter.context.defined_functions) > 0
    assert "f_rsi_cross" in converter.context.defined_functions


def test_pine_script_config():
    """Test PineScript configuration options"""
    config = PineScriptConfig(
        version="5",
        timeframe="H",
        overlay=False,
        precision=6,
        max_bars_back=1000,
        additional_imports=["import talib.abstract as abstract"],
    )

    converter = PineScriptConverter(config)
    assert converter.config.version == "5"
    assert converter.config.timeframe == "H"
    assert not converter.config.overlay
    assert converter.config.precision == 6
    assert converter.config.max_bars_back == 1000
    assert "import talib.abstract as abstract" in converter.config.additional_imports


def test_builtin_function_conversion(converter):
    """Test conversion of PineScript built-in functions"""
    pine_script = """
    //@version=5
    strategy("Technical Indicators")
    
    rsi_value = rsi(close, 14)
    macd_line, signal, hist = macd(close, 12, 26, 9)
    highest_high = highest(high, 20)
    lowest_low = lowest(low, 20)
    """

    python_code = converter.convert(pine_script)

    # Check for correct Python equivalents
    assert "ta.RSI" in python_code
    assert "ta.MACD" in python_code
    assert "ta.highest" in python_code
    assert "ta.lowest" in python_code


def test_variable_mapping(converter):
    """Test conversion of PineScript variables to Python DataFrame columns"""
    pine_script = """
    //@version=5
    strategy("Price Action")
    
    price_range = high - low
    typical_price = (high + low + close) / 3
    volume_price = close * volume
    """

    python_code = converter.convert(pine_script)

    # Check for correct DataFrame column references
    assert 'df["High"]' in python_code
    assert 'df["Low"]' in python_code
    assert 'df["Close"]' in python_code
    assert 'df["Volume"]' in python_code


def test_complex_strategy(converter):
    """Test conversion of a complex strategy with multiple components"""
    pine_script = """
    //@version=5
    strategy("Complex Strategy", overlay=true)
    
    // Inputs
    rsi_length = input(14, "RSI Length")
    rsi_oversold = input(30, "RSI Oversold")
    rsi_overbought = input(70, "RSI Overbought")
    
    // Calculations
    rsi_value = rsi(close, rsi_length)
    ema_20 = ema(close, 20)
    ema_50 = ema(close, 50)
    
    // Conditions
    trend_up = ema_20 > ema_50
    rsi_buy = rsi_value < rsi_oversold
    rsi_sell = rsi_value > rsi_overbought
    
    // Entry/Exit rules
    if (trend_up and rsi_buy)
        strategy.entry("RSI_EMA_Long", strategy.long)
    if (rsi_sell)
        strategy.close("RSI_EMA_Long")
    """

    python_code = converter.convert(pine_script)

    # Check for complex strategy components
    assert "class ComplexStrategy:" in python_code
    assert "rsi_length" in python_code
    assert "rsi_oversold" in python_code
    assert "rsi_overbought" in python_code
    assert "ta.RSI" in python_code
    assert "ta.EMA" in python_code
    assert "calculate_signals" in python_code
