"""Tests for the research agent system."""

import pytest
import pandas as pd
import numpy as np
import torch
from datetime import datetime, timezone
from research_agent.main import ResearchAgent
from research_agent.knowledge_graph import KnowledgeGraph
from app.core.event_bus import Event, EventType, EventBus
import redis
from unittest.mock import AsyncMock, patch


@pytest.fixture
def event_bus():
    """Create event bus for testing."""
    redis_client = redis.Redis(host="localhost", port=6379, db=0)
    try:
        yield EventBus(redis_client=redis_client)
    finally:
        redis_client.close()


@pytest.fixture
async def research_agent(event_bus):
    """Create research agent for testing."""
    agent = ResearchAgent(event_bus, None)
    yield agent


@pytest.fixture
def sample_market_data():
    """Create sample market data for testing."""
    dates = pd.date_range(
        start=datetime(2024, 1, 1, tzinfo=timezone.utc),
        end=datetime(2024, 1, 10, tzinfo=timezone.utc),
        freq="1min",
    )

    data = pd.DataFrame(
        {
            "open": np.random.normal(100, 1, len(dates)),
            "high": np.random.normal(101, 1, len(dates)),
            "low": np.random.normal(99, 1, len(dates)),
            "close": np.random.normal(100, 1, len(dates)),
            "volume": np.random.normal(1000, 100, len(dates)),
        },
        index=dates,
    )

    return data


@pytest.mark.asyncio
async def test_market_data_processing(event_bus, sample_market_data):
    """Test market data processing pipeline."""
    event = Event(
        type=EventType.MARKET_DATA,
        payload={"symbol": "BTC-USD", "data": sample_market_data.to_dict()},
    )

    with patch("research_agent.main.ResearchAgent") as mock_agent:
        mock_agent.process_market_data = AsyncMock()
        await event_bus.publish(event)
        mock_agent.process_market_data.assert_called_once()


@pytest.mark.asyncio
async def test_anomaly_detection(event_bus, sample_market_data):
    """Test anomaly detection in market data."""
    # Inject anomaly
    anomaly_idx = len(sample_market_data) // 2
    sample_market_data.iloc[anomaly_idx, :] *= 10

    event = Event(
        type=EventType.MARKET_DATA,
        payload={"symbol": "BTC-USD", "data": sample_market_data.to_dict()},
    )

    with patch("research_agent.main.ResearchAgent") as mock_agent:
        mock_agent.detect_anomalies = AsyncMock()
        await event_bus.publish(event)
        mock_agent.detect_anomalies.assert_called_once()


@pytest.mark.asyncio
async def test_order_book_processing(event_bus):
    """Test order book processing."""
    book_data = {
        "bids": [[100.0, 1.5], [99.5, 2.0], [99.0, 1.0]],
        "asks": [[100.5, 1.0], [101.0, 2.0], [101.5, 1.5]],
        "timestamp": datetime.now(timezone.utc).timestamp(),
    }

    event = Event(
        type=EventType.ORDER_BOOK, payload={"symbol": "BTC-USD", "data": book_data}
    )

    with patch("research_agent.main.ResearchAgent") as mock_agent:
        mock_agent.process_order_book = AsyncMock()
        await event_bus.publish(event)
        mock_agent.process_order_book.assert_called_once()


@pytest.mark.asyncio
async def test_knowledge_graph_integration(event_bus):
    """Test knowledge graph functionality."""
    test_data = {
        "market_state": {
            "trend": "bullish",
            "volatility": "high",
            "liquidity": "moderate",
        },
        "technical_indicators": {"rsi": 65, "macd": 0.5, "bb_width": 2.1},
        "fundamental_data": {"market_cap": 1e9, "volume_24h": 1e6, "dominance": 45},
    }

    event = Event(type=EventType.KNOWLEDGE_UPDATE, payload=test_data)

    with patch("research_agent.main.ResearchAgent") as mock_agent:
        mock_agent.update_knowledge = AsyncMock()
        await event_bus.publish(event)
        mock_agent.update_knowledge.assert_called_once()


@pytest.mark.asyncio
async def test_model_updates(event_bus, sample_market_data):
    """Test model training and updates."""
    event = Event(
        type=EventType.MODEL_UPDATE,
        payload={"symbol": "BTC-USD", "data": sample_market_data.to_dict()},
    )

    with patch("research_agent.main.ResearchAgent") as mock_agent:
        mock_agent.update_model = AsyncMock()
        await event_bus.publish(event)
        mock_agent.update_model.assert_called_once()
