"""
Test suite for backtesting validation modules
"""

import unittest
import numpy as np
import pandas as pd
from datetime import datetime, timedelta
from validation.walk_forward import WalkForwardAnalyzer, WalkForwardConfig
from validation.monte_carlo import MonteCarloSimulator, MonteCarloConfig


class TestWalkForwardAnalysis(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        """Create sample data for testing"""
        dates = pd.date_range(start="2024-01-01", end="2024-12-31", freq="D")
        np.random.seed(42)

        # Generate synthetic price data
        prices = 100 * (1 + np.random.normal(0.0001, 0.02, len(dates)).cumsum())

        cls.test_data = pd.DataFrame(
            {
                "timestamp": dates,
                "open": prices * (1 + np.random.normal(0, 0.002, len(dates))),
                "high": prices * (1 + np.random.normal(0.002, 0.002, len(dates))),
                "low": prices * (1 + np.random.normal(-0.002, 0.002, len(dates))),
                "close": prices,
                "volume": np.random.lognormal(10, 1, len(dates)),
            }
        ).set_index("timestamp")

    def test_walk_forward_initialization(self):
        """Test WalkForwardAnalyzer initialization"""
        analyzer = WalkForwardAnalyzer()
        self.assertIsNotNone(analyzer)
        self.assertIsInstance(analyzer.config, WalkForwardConfig)

    def test_anchored_analysis(self):
        """Test anchored walk-forward analysis"""
        analyzer = WalkForwardAnalyzer()
        results = analyzer.analyze(self.test_data)

        self.assertIn("anchored", results)
        self.assertIn("rolling", results)
        self.assertIn("metrics", results)

        # Check stability metrics
        self.assertGreaterEqual(results["metrics"]["overall_stability"], 0)
        self.assertLessEqual(results["metrics"]["overall_stability"], 1)

    def test_performance_metrics(self):
        """Test performance metric calculations"""
        analyzer = WalkForwardAnalyzer()
        window_metrics = analyzer._calculate_window_metrics(self.test_data)

        self.assertIn("sharpe", window_metrics)
        self.assertIn("sortino", window_metrics)
        self.assertIn("max_drawdown", window_metrics)
        self.assertIn("win_rate", window_metrics)
        self.assertIn("profit_factor", window_metrics)


class TestMonteCarloSimulation(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        """Create sample data for testing"""
        dates = pd.date_range(start="2024-01-01", end="2024-12-31", freq="D")
        np.random.seed(42)

        # Generate synthetic price data with some volatility clustering
        returns = np.random.normal(0.0001, 0.02, len(dates))
        volatility = np.abs(np.random.normal(0, 0.01, len(dates))).cumsum()
        returns = returns * (1 + volatility)
        prices = 100 * (1 + returns).cumprod()

        cls.test_data = pd.DataFrame(
            {
                "timestamp": dates,
                "open": prices * (1 + np.random.normal(0, 0.002, len(dates))),
                "high": prices * (1 + np.random.normal(0.002, 0.002, len(dates))),
                "low": prices * (1 + np.random.normal(-0.002, 0.002, len(dates))),
                "close": prices,
                "volume": np.random.lognormal(10, 1, len(dates)),
            }
        ).set_index("timestamp")

    def test_monte_carlo_initialization(self):
        """Test MonteCarloSimulator initialization"""
        simulator = MonteCarloSimulator()
        self.assertIsNotNone(simulator)
        self.assertIsInstance(simulator.config, MonteCarloConfig)

    def test_path_generation(self):
        """Test different path generation methods"""
        simulator = MonteCarloSimulator(
            MonteCarloConfig(n_simulations=100)  # Reduced for testing
        )

        for generator in ["gaussian", "bootstrap", "historical"]:
            paths = simulator._generate_paths(self.test_data, generator)

            self.assertEqual(paths.shape[0], 100)  # n_simulations
            self.assertEqual(paths.shape[1], len(self.test_data) - 1)  # n_days
            self.assertTrue(np.all(paths > 0))  # All prices should be positive

    def test_black_swan_simulation(self):
        """Test black swan event simulation"""
        simulator = MonteCarloSimulator(
            MonteCarloConfig(n_simulations=100)  # Reduced for testing
        )

        results = simulator._simulate_black_swans(self.test_data)

        self.assertIn("paths", results)
        self.assertIn("metrics", results)
        self.assertIn("events", results)

        # Check if events include expected scenarios
        self.assertIn("market_crash", results["events"])
        self.assertIn("flash_crash", results["events"])
        self.assertIn("volatility_spike", results["events"])

    def test_risk_metrics(self):
        """Test risk metric calculations"""
        simulator = MonteCarloSimulator(
            MonteCarloConfig(n_simulations=100)  # Reduced for testing
        )

        paths = simulator._generate_paths(self.test_data, "gaussian")
        metrics = simulator._calculate_path_metrics(paths)
        risk_metrics = simulator._calculate_risk_metrics(paths, metrics)

        self.assertIn("tail_ratio", risk_metrics)
        self.assertIn("stability_ratio", risk_metrics)
        self.assertIn("calmar_ratio", risk_metrics)
        self.assertIn("ulcer_index", risk_metrics)
        self.assertIn("pain_ratio", risk_metrics)


if __name__ == "__main__":
    unittest.main()
